@@ -1,36 +1,43 @@
 package Mojo::Webqq::Group;
 use strict;
 use List::Util qw(first);
-use base qw(Mojo::Base Mojo::Webqq::Model::Base);
-sub has { Mojo::Base::attr(__PACKAGE__, @_) };
+use Mojo::Webqq::Group::Member;
+use Mojo::Webqq::Base 'Mojo::Webqq::Model::Base';
 has [qw(
-    gid
-    gcode
-    gtype 
-    gnumber
-    gname
-    gmemo
-    gcreatetime
-    glevel
-    gowner
-    gmarkname    
+    id
+    uid
+    code
+    role
+    name
+    memo
+    createtime
+    level
+    owner_id
+    owner_uid
+    markname
+    max_member
+    max_admin
 )];
 has member => sub{[]};
-sub id {
+sub member_count {0+@{$_[0]->member}}
+sub number {$_[0]->uid}
+sub AUTOLOAD {
     my $self = shift;
-    return @_?$self->gid(@_):$self->gid;
-}
-sub name{
-    my $self = shift;
-    return @_?$self->gname(@_):$self->gname;
-}
-sub markname{
-    my $self = shift;
-    return @_?$self->gmarkname(@_):$self->gmarkname;
+    if($Mojo::Webqq::Group::AUTOLOAD =~ /.*::gnumber/){
+        $self->uid(@_);
+    }
+    elsif($Mojo::Webqq::Group::AUTOLOAD =~ /.*::gtype/){
+        $self->role(@_);
+    }
+    elsif($Mojo::Webqq::Group::AUTOLOAD =~ /.*::g(.*)/){
+        my $attr = $1;
+        $self->$attr(@_);
+    }
+    else{die("undefined subroutine $Mojo::Webqq::Group::AUTOLOAD");}
 }
 sub displayname {
     my $self = shift;
-    return defined $self->gmarkname?$self->gmarkname:$self->gname;
+    return defined $self->markname?$self->markname:$self->name;
 }
 sub new {
     my $class = shift;
@@ -38,7 +45,8 @@ sub new {
     bless $self=@_ ? @_ > 1 ? {@_} : {%{$_[0]}} : {}, ref $class || $class;
     if(exists $self->{member} and ref $self->{member} eq "ARRAY"){
         for( @{ $self->{member} } ){
-            $_ = $self->{_client}->new_group_member($_) if ref $_ ne "Mojo::Webqq::Group::Member";
+            $_->{_group_id} = $self->id if not defined $_->{_group_id};
+            $_ = Mojo::Webqq::Group::Member->new($_) if ref $_ ne "Mojo::Webqq::Group::Member";
         } 
     }
     $self;
@@ -47,23 +55,23 @@ sub new {
 sub each_group_member{
     my $self = shift;
     my $callback = shift;
-    $self->{_client}->die("参数必须是函数引用") if ref $callback ne "CODE";
+    $self->client->die("参数必须是函数引用") if ref $callback ne "CODE";
     return if ref $self->member ne "ARRAY";
-    $self->{_client}->update_group($self) if $self->is_empty;
+    $self->client->update_group($self) if $self->is_empty;
     for(@{$self->member}){
-        $callback->($self->{_client},$_); 
+        $callback->($self->client,$_); 
     }
 }
 sub members {
     my $self = shift;
-    $self->{_client}->update_group($self) if $self->is_empty;
+    $self->client->update_group($self) if $self->is_empty;
     return @{$self->member};
 }
 sub search_group_member{
     my $self = shift;
     my %p = @_;
     return if 0 == grep {defined $p{$_}} keys %p;
-    $self->{_client}->update_group($self) if $self->is_empty;
+    $self->client->update_group($self) if $self->is_empty;
     if(wantarray){
         return grep {my $m = $_;(first {$p{$_} ne $m->$_} grep {defined $p{$_}} keys %p) ? 0 : 1;} @{$self->member};
     }
@@ -76,7 +84,7 @@ sub add_group_member{
     my $self = shift;   
     my $member = shift;
     my $nocheck = shift;
-    $self->{_client}->die("不支持的数据类型") if ref $member ne "Mojo::Webqq::Group::Member";
+    $self->client->die("不支持的数据类型") if ref $member ne "Mojo::Webqq::Group::Member";
     if($nocheck){
         push @{$self->member},$member;
         return $self;
@@ -94,7 +102,7 @@ sub add_group_member{
 sub remove_group_member{
     my $self = shift;
     my $member = shift;
-    $self->{_client}->die("不支持的数据类型") if ref $member ne "Mojo::Webqq::Group::Member";
+    $self->client->die("不支持的数据类型") if ref $member ne "Mojo::Webqq::Group::Member";
     for(my $i=0;$i<@{$self->member};$i++){
         if($self->member->[$i]->id eq $member->id){
             splice @{$self->member},$i,1;
@@ -111,11 +119,11 @@ sub is_empty{
 
 sub update_group_member_ext {
     my $self = shift;
-    $self->{_client}->update_group_member_ext($self,@_);
+    $self->client->update_group_member_ext($self,@_);
 }
 sub update_group_member {
     my $self = shift;
-    $self->{_client}->update_group_member($self,@_);
+    $self->client->update_group_member($self,@_);
 }
 sub update{
     my $self = shift;
@@ -123,19 +131,19 @@ sub update{
     for(grep {substr($_,0,1) ne "_"} keys %$hash){
         if($_ eq "member" and ref $hash->{member} eq "ARRAY"){
             next if not @{$hash->{member}};
-            my @member = map {ref $_ eq "Mojo::Webqq::Group::Member"?$_:$self->{_client}->new_group_member($_)} @{$hash->{member}};
+            my @member = map {ref $_ eq "Mojo::Webqq::Group::Member"?$_:Mojo::Webqq::Group::Member->new($_)} @{$hash->{member}};
             if( $self->is_empty() ){
                 $self->member(\@member);
             }
             else{
-                my($new_members,$lost_members,$sames)=$self->{_client}->array_diff($self->member, \@member,sub{$_[0]->id});
+                my($new_members,$lost_members,$sames)=$self->client->array_diff($self->member, \@member,sub{$_[0]->id});
                 for(@{$new_members}){
                     $self->add_group_member($_);
-                    $self->{_client}->emit(new_group_member=>$_) if defined $self->{_client};
+                    $self->client->emit(new_group_member=>$_);
                 }
                 for(@{$lost_members}){
                     $self->remove_group_member($_);
-                    $self->{_client}->emit(lose_group_member=>$_) if defined $self->{_client};
+                    $self->client->emit(lose_group_member=>$_);
                 }
                 for(@{$sames}){
                     my($old_member,$new_member) = ($_->[0],$_->[1]);
@@ -151,14 +159,14 @@ sub update{
                         my $old_property = $self->{$_};
                         my $new_property = $hash->{$_};
                         $self->{$_} = $hash->{$_};
-                        $self->{_client}->emit("group_property_change"=>$self,$_,$old_property,$new_property) if defined $self->{_client};
+                        $self->client->emit("group_property_change"=>$self,$_,$old_property,$new_property);
                     }
                 }
                 elsif( ! (!defined $hash->{$_} and !defined $self->{$_}) ){
                     my $old_property = $self->{$_};
                     my $new_property = $hash->{$_};
                     $self->{$_} = $hash->{$_};
-                    $self->{_client}->emit("group_property_change"=>$self,$_,$old_property,$new_property) if defined $self->{_client};
+                    $self->client->emit("group_property_change"=>$self,$_,$old_property,$new_property);
                 }
             }
         }
@@ -168,51 +176,51 @@ sub update{
 
 sub send {
     my $self = shift;
-    $self->{_client}->send_group_message($self,@_);
+    $self->client->send_group_message($self,@_);
 } 
 sub me {
     my $self = shift;
-    $self->search_group_member(id=>$self->{_client}->user->id);
+    $self->search_group_member(id=>$self->client->user->id);
 }
 sub invite_friend{
     my $self = shift;
     my @friends = @_;
-    return $self->{_client}->invite_friend($self,@friends);
+    return $self->client->invite_friend($self,@friends);
 }
 sub shutup_group_member{
     my $self = shift;
     my $time = shift;
     my @members = @_;
-    return $self->{_client}->shutup_group_member($self,$time,@members);
+    return $self->client->shutup_group_member($self,$time,@members);
 }
 sub speakup_group_member{
     my $self = shift;
     my @members = @_;
-    return $self->{_client}->speakup_group_member($self,@members);
+    return $self->client->speakup_group_member($self,@members);
 }
 sub kick_group_member{
     my $self = shift;
     my @members = @_;
-    return $self->{_client}->kick_group_member($self,@members);
+    return $self->client->kick_group_member($self,@members);
 }
 sub set_group_admin{
     my $self = shift;
     my @members = @_;
-    return $self->{_client}->set_group_admin($self,@members);
+    return $self->client->set_group_admin($self,@members);
 }
 sub remove_group_admin{
     my $self = shift;
     my @members = @_;
-    return $self->{_client}->remove_group_admin($self,@members);
+    return $self->client->remove_group_admin($self,@members);
 }
 sub set_group_member_card{
     my $self = shift;
     my $member = shift;
     my $card = shift;
-    return $self->{_client}->set_group_member_card($self,$member,$card);
+    return $self->client->set_group_member_card($self,$member,$card);
 }
 sub qiandao {
     my $self = shift;
-    return $self->{_client}->qiandao($self);
+    return $self->client->qiandao($self);
 }
 1;
